#include <string.h>
#include "error.h"
#include "config.h"
#include "loadfile.h"
#include "reffile.h"
#include "cdbslib.h"
#include "cdbserror.h"
#include "expload.h"
#include "mkload.h"

//===================================================================
//
// Name: mkload_expload
//
// Description: Expand any wildcard keywords in a loadfile table. 
//
// History:
// --------
// 06/08/11 68503 MSwam     first version
//
//===================================================================

void mkload_expload(table **loadfile_table, // io :loadfile table descriptor ptr
                    char *instrument      // i : instrument for this ref file
                    )
{
    // Determine if expload is needed
    if (!mkload_expload_necessary(*loadfile_table, instrument)) {
       return;
    }

    fprintf(stderr,
      "\nThe EXPLOAD tool is being run to expand existing wildcard values...\n");

    // Get the expload rules filename for this instrument
    char expload_rulesfile[256];
    mkload_expload_rules_filename(instrument, expload_rulesfile);

    // Create output loadfile with the same layout and header as the input one,
    // but add the EXPANSION_NUMBER row
    //
    table *expload_table = create_table();
    copy_layout (*loadfile_table, expload_table, "EXPANSION_NUMBER");
    copy_header (*loadfile_table, expload_table);

    // Create the target and action table from the rules file.
    // The target table contains the fields that match the input table.
    // The action table contains the rows used to expand a row in the
    // input table when a match is found.
     
    table *target = create_table ();
    table *action = create_table ();

    parse_rules (expload_rulesfile, *loadfile_table, target, action);

    // Use the target and action tables to expand the rows in the loadfile
    // table into the expload table
    use_rules (*loadfile_table, expload_table, target, action);

    free_table (target);
    free_table (action);

    // return the exploaded table in place of the original 
    // (free up the original)
    //
    free_table (*loadfile_table);
    *loadfile_table = expload_table;

    return;
}
